#include <iostream>
#include "dlgmodulelist.h"
#include "globals.h"
#include <qlabel.h>

using namespace std;

DlgModuleList::DlgModuleList(QWidget *parent, const char *name, WFlags f)
		: QWidget(parent, name, f) {
	layoutMain = new QVBoxLayout(this, 0, -1, "vertical layout");
	moduleList = new QListView(this, "module list box");
	layoutBtns = new QHBox(this, "horizontal layout");
	motdBox = new QTextEdit(this, "message of the day text box");
	btnOK = new QPushButton("Choose", layoutBtns, "choose button");
	btnCancel = new QPushButton("Cancel", layoutBtns, "cancel button");
	QLabel *motdLabel = new QLabel("  Message of the day", this, "motd label");

	this->resize(500,300);
	this->setCaption("Choose a remote module");

	layoutMain->addWidget(moduleList);
	layoutMain->addWidget(motdLabel);
	layoutMain->addWidget(motdBox);
	layoutMain->addWidget(layoutBtns);
	moduleList->addColumn("Module Name");
	moduleList->addColumn("Description");
	
	motdBox->setReadOnly(true);
	btnOK->setMaximumWidth(100);
	btnCancel->setMaximumWidth(100);

	connect(moduleList, SIGNAL(selectionChanged()), this, SLOT(updateStatus()));
	connect(moduleList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(listDoubleClicked(QListViewItem *)));
	connect(btnOK, SIGNAL(clicked()), this, SLOT(tryFinish()));
	connect(btnCancel, SIGNAL(clicked()), this, SLOT(cancel()));

	updateStatus();
}

void DlgModuleList::addModule(QString name, QString description) {
	new QListViewItem(moduleList, name, description);
}

void DlgModuleList::clearList() {
	moduleList->clear();
}

void DlgModuleList::setMotd(QStringList motd) {
	motdBox->clear();
	for (QStringList::iterator it = motd.begin(); it != motd.end(); it++)
		if ("" == *it)
			motdBox->append("\n");
		else
			motdBox->append(*it);
}

void DlgModuleList::updateStatus() {
	if (!moduleList->selectedItem())
		btnOK->setEnabled(false);
	else {
		btnOK->setEnabled(true);
	}
}

void DlgModuleList::listDoubleClicked(QListViewItem *item) {
	if (item) tryFinish();
}

void DlgModuleList::tryFinish() {
	emit moduleChosen((*moduleList->selectedItem()).text(0));
	this->close();
	emit moduleListClosed();
}

void DlgModuleList::cancel() {
	this->close();
	emit moduleListClosed();
}
