#include "globals.h"
#include "dlgstatus.h"
#include "../paths.h"
#include <iostream>
#include "customevents.h"
#include <qapplication.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qthread.h>
#include <qtooltip.h>
#include <qpixmap.h>

using namespace std;

// global pointers used for event passing in syncthread.cpp

DlgStatus *statusDlgPtr = NULL;
SyncProcess *SyncProcessPtr = NULL;
SiteInfo *siteInfoPtr = NULL;

DlgStatus::DlgStatus(SiteInfo &si, QWidget *parent, const char *name)
		: QWidget(parent, name) {
	// various nested layouts and widgets
	gridMain        = new QGridLayout(this, 4, 1);
	listPanel       = new QWidget(this, "panel holding file/transfer lists");
	splitterMain    = new QSplitter(Qt::Vertical, listPanel, "main widget splitter");
	// these 2 surround the splitter bar
	topListCont     = new QWidget(splitterMain, "nested list + button container");
	transferList    = new QListView(splitterMain, "transferrable file list");
	// these go in the "top list container", which takes up most of the dialog
	gridLists       = new QGridLayout(topListCont, 2, 5, 4); // file listviews
	leftList        = new QListView(topListCont, "left side file box");
	rightList       = new QListView(topListCont, "right side file box");
	operationsPanel = new QVBox(topListCont, "operations box");
	localPathBox    = new QLineEdit(topListCont);
	remotePathBox   = new QLineEdit(topListCont);
	localLabel      = new QLabel(localPathBox, "&Local:", topListCont);
	remoteLabel     = new QLabel(remotePathBox, "&Remote:", topListCont);
	// top text fields
	connFieldPanel = new QWidget(this, "panel holding connection info fields");
	connFieldBox   = new QHBoxLayout(connFieldPanel, 6);
	hostnameBox    = new QLineEdit(connFieldPanel);
	portBox        = new QLineEdit(connFieldPanel);
	usernameBox    = new QLineEdit(connFieldPanel);
	passwordBox    = new QLineEdit(connFieldPanel);
	hostnameLabel  = new QLabel(hostnameBox, "&Host:", connFieldPanel);
	portLabel      = new QLabel(portBox, "Por&t:", connFieldPanel);
	usernameLabel  = new QLabel(usernameBox, "&Username:", connFieldPanel);
	passwordLabel  = new QLabel(passwordBox, "&Password:", connFieldPanel);
	// Miscellaneous crapola
	fileProgress  = new QProgressBar(100, this, "file progress bar");
	statusBar     = new QStatusBar(this, "status bar");
	imgDownload   = new QPixmap(QString("%1/share/pixmaps/qsync/arrowLeft.png").arg(PREFIX));
	imgUpload     = new QPixmap(QString("%1/share/pixmaps/qsync/arrowRight.png").arg(PREFIX));
	btnDownload   = new QPushButton("<==", operationsPanel, "download button");
	btnUpload     = new QPushButton("==>", operationsPanel, "upload button");
	btnStart      = new QPushButton("&Start/Refresh", connFieldPanel, "start connection button");
	localDir      = new QDir();
	siteInfo      = &si;
	moduleListDlg = new DlgModuleList(0, "Module list dialog", Qt::WShowModal);
	syncProcess   = new SyncProcess(*siteInfo, *this, *moduleListDlg);
	moduleListDlg->hide();
	nameHeader    = "Name";
	permsHeader   = "Perms";
	originalDir   = localDir->absPath();

	// setup global pointers used in callbacks
	statusDlgPtr   = this;
	SyncProcessPtr = syncProcess;
	siteInfoPtr    = &si;

	connFieldBox->addWidget(hostnameLabel);
	connFieldBox->addWidget(hostnameBox);
	connFieldBox->setStretchFactor(hostnameBox, 10);
	connFieldBox->addWidget(portLabel);
	connFieldBox->addWidget(portBox);
	connFieldBox->setStretchFactor(portBox, 3);
	connFieldBox->addWidget(usernameLabel);
	connFieldBox->addWidget(usernameBox);
	connFieldBox->setStretchFactor(usernameBox, 7);
	connFieldBox->addWidget(passwordLabel);
	connFieldBox->addWidget(passwordBox);
	connFieldBox->setStretchFactor(passwordBox, 7);
	connFieldBox->addWidget(btnStart);

	gridLists->addWidget(localLabel, 0, 0);
	gridLists->addWidget(localPathBox, 0, 1);
	gridLists->addMultiCellWidget(leftList, 1, 1, 0, 1);
	gridLists->addMultiCellWidget(operationsPanel, 1, 1, 2, 2);
	gridLists->addWidget(remoteLabel, 0, 3);
	gridLists->addWidget(remotePathBox, 0, 4);
	gridLists->addMultiCellWidget(rightList, 1, 1, 3, 4);

	// make file list columns stretchable
	gridLists->setColStretch(1, 10);
	gridLists->setColStretch(4, 10);
	// make file/transfer panel row stretchable
	gridMain->setRowStretch(1, 10);

	leftList->show();
	rightList->show();
	operationsPanel->show();
	splitterMain->show();
	gridMain->addWidget(connFieldPanel, 0, 0);
	gridMain->addWidget(listPanel, 1, 0);
	gridMain->addWidget(fileProgress, 2, 0);
	gridMain->addWidget(statusBar, 3, 0);
	rightList->addColumn(nameHeader);
	rightList->addColumn("Size");
	rightList->addColumn("Time");
	rightList->addColumn("Date");
	rightList->addColumn(permsHeader);
	leftList->addColumn(nameHeader);
	leftList->addColumn("Size");
	leftList->addColumn("Time");
	leftList->addColumn("Date");
	leftList->addColumn(permsHeader);
	transferList->addColumn("Filename");
	transferList->addColumn("Size");
	transferList->setColumnAlignment(1, Qt::AlignRight);
	transferList->addColumn("%");
	transferList->addColumn("Transfer Rate");
	transferList->setColumnAlignment(3, Qt::AlignRight);
	transferList->addColumn("Time");

	// shit moved over from old main dialog (other shit scattered elsewhere)
	connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));
	// TODO: get rid of this soon - don't hide this dialog, but make mod list modal.
	connect(syncProcess, SIGNAL(syncDone()), this, SLOT(refreshLocal()));
	connect(moduleListDlg, SIGNAL(moduleChosen(QString)), this, SLOT(remoteChosen(QString)));
	connect(btnStart, SIGNAL(clicked()), this, SLOT(runSync()));
	connect(hostnameBox, SIGNAL(textChanged(const QString &)), this, SLOT(changeHost(const QString &)));
	connect(portBox, SIGNAL(textChanged(const QString &)), this, SLOT(changePort(const QString &)));
	connect(usernameBox, SIGNAL(textChanged(const QString &)), this, SLOT(changeUser(const QString &)));
	connect(passwordBox, SIGNAL(textChanged(const QString &)), this, SLOT(changePass(const QString &)));
	// disabled, why update localDir on every text change?  possibly useful for storing site info at some point... consider deletable in a while (today: 20051217)
	//connect(localPathBox, SIGNAL(textChanged(const QString &)), this, SLOT(changeLocal(const QString &)));
	connect(localPathBox, SIGNAL(returnPressed()), this, SLOT(localDirEntered()));
	connect(remotePathBox, SIGNAL(textChanged(const QString &)), this, SLOT(changeRemote(const QString &)));
	connect(remotePathBox, SIGNAL(returnPressed()), this, SLOT(remoteDirEntered()));
	// obsolete: this IS the xferDlg now.  eliminate signal and then this part.
	// connect(syncProcess, SIGNAL(transferDialogShown()), this, SLOT(hide()));
	connect(rightList, SIGNAL(selectionChanged()), this, SLOT(remoteSelectChanged()));
	connect(rightList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(remoteListDoubleClicked(QListViewItem *)));
	connect(rightList, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(remoteListDoubleClicked(QListViewItem *)));
	connect(leftList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(localListDoubleClicked(QListViewItem *)));
	connect(leftList, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(localListDoubleClicked(QListViewItem *)));
	connect(btnDownload, SIGNAL(clicked()), this, SLOT(downloadClicked()));
	connect(btnUpload, SIGNAL(clicked()), this, SLOT(uploadClicked()));

	leftList->setShowSortIndicator(true);
	rightList->setShowSortIndicator(true);
	transferList->setShowSortIndicator(true);
	// also stuff pulled from old main dialog
	passwordBox->setEchoMode(QLineEdit::Password);
	portBox->setMaxLength(PORT_LENGTH);
	portBox->setText(siteInfo->getPort().c_str());
	localPathBox->setText(localDir->absPath());
	btnUpload->setEnabled(false);
	btnDownload->setPixmap(*imgDownload);
	btnUpload->setPixmap(*imgUpload);
	QToolTip::add(btnDownload, "<-- ( Download )");
	QToolTip::add(btnUpload, "--> ( Upload )");
	btnDownload->setFixedSize(imgDownload->width() + 20, imgDownload->height() + 20);
	btnUpload->setFixedSize(imgUpload->width() + 20, imgUpload->height() + 20);
	fillLocalList();
	remoteSelectChanged();
	this->resize(800, 400); // FIXME: choose reasonable size based on res
}

void DlgStatus::resizeEvent(QResizeEvent *e) {
	QRect cellSize = gridMain->cellGeometry(1, 0);
	splitterMain->resize(cellSize.width(), cellSize.height());
	recenterModDlg();
}

void DlgStatus::moveEvent(QMoveEvent *e) {
	recenterModDlg();
}

void DlgStatus::recenterModDlg() {
	QRect sizeThis  = frameGeometry();
	QRect sizeChild = moduleListDlg->frameGeometry();
	int centerX = sizeThis.x() + (sizeThis.width()  / 2);
	int centerY = sizeThis.y() + (sizeThis.height() / 2);
	int childX  = centerX - (sizeChild.width()  / 2);
	int childY  = centerY - (sizeChild.height() / 2);
	moduleListDlg->move(childX, childY);
}

void DlgStatus::customEvent(QCustomEvent *e) {
	QStringList *args;
	QString data;
	switch (e->type()) {
		case EVENT_ADD_REMOTE:
			args = (QStringList *)e->data();
			addListItem(rightList, (*args)[0], (*args)[1], (*args)[2], (*args)[3]);
			break;
		case EVENT_SET_CURRENT:
			data = (char *)e->data();
			setCurrFile(data);
			break;
		case EVENT_SET_PROGRESS:
			data = (char *)e->data();
			setFileProgress(data);
			break;
		case EVENT_QUEUE_XFER:
			data = (char *)e->data();
			addTransferItem(data);
			break;
		default:
			cerr << "ERROR: bad customEvent \"" << e->type() << "\" in " << __FILE__ << ":" << __LINE__ << ".  You should never see this!" << endl;
			break;
	}
}

void DlgStatus::fillLocalList() {
	leftList->clear();
	QString perms;
	QFileInfoList *fileList = (QFileInfoList *) localDir->entryInfoList();
	for (QFileInfo *fInfo = fileList->first(); fInfo; fInfo = fileList->next()) {
		if (fInfo->fileName() != ".") {
			if ("/" == localDir->absPath())
				if (".." == fInfo->fileName())
					continue;
			// time to list all permission bits
			if (fInfo->isSymLink()) perms = "l";
			else if (fInfo->isDir()) perms = "d";
			else perms = "-";
			if (fInfo->permission(QFileInfo::ReadUser)) perms += "r";
			else perms += "-";
			if (fInfo->permission(QFileInfo::WriteUser)) perms += "w";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ExeUser)) perms += "x";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ReadGroup)) perms += "r";
			else perms += "-";
			if (fInfo->permission(QFileInfo::WriteGroup)) perms += "w";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ExeGroup)) perms += "x";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ReadOther)) perms += "r";
			else perms += "-";
			if (fInfo->permission(QFileInfo::WriteOther)) perms += "w";
			else perms += "-";
			// Can't use QFileInfo::permission(int) for sticky bit.
			if (fInfo->permission(QFileInfo::ExeOther)) perms += "x";
			else perms += "-"; // Finally, done!  Condense this better later.
			addListItem(leftList, perms, QString("%1").arg(fInfo->size()),
				fInfo->lastModified().date().toString() + "*" + fInfo->lastModified().time().toString(), fInfo->fileName(), true);
		}
	}
}

// dates added from fillLocalList currently are in a different format.  have them split on special char "*".
void DlgStatus::addListItem(QListView *list, const char *perms, const char *size, const char *modtime, const char *name, bool dateWeird) {
	if (strcmp(".", name)) {
		QString timeDate = modtime;
		QListViewItem *item = new QListViewItem(list, name, size, timeDate.section(dateWeird ? '*' : ' ', 1, 1), timeDate.section(dateWeird ? '*' : ' ', 0, 0), perms);
		// add icons - courtesy of HPV
    if (item->text(4)[0] == 'd' || item->text(4)[0] == 't') {
			if (item->text(0) == "..")
				item->setPixmap(0, QPixmap(QString("%1/share/pixmaps/qsync/arrowUp.png").arg(PREFIX)));
			else
				item->setPixmap(0, QPixmap(QString("%1/share/pixmaps/qsync/folder.png").arg(PREFIX)));
		}
		else
			item->setPixmap(0, QPixmap(QString("%1/share/pixmaps/qsync/file.png").arg(PREFIX)));
	}
}

void DlgStatus::addTransferItem(QString name) {
	new QListViewItem(transferList, name);
}

void DlgStatus::localListDoubleClicked(QListViewItem *item) {
	QString temp = localDir->absPath() + "/" + item->text(0);
	if (QFileInfo(temp).isDir() && !QFileInfo(temp).isSymLink()) {
		localDir->cd(temp);
		localPathBox->setText(localDir->absPath());
		leftList->clear();
		fillLocalList();
	}
	else
		emit syncSendFile(temp.remove(0, temp.findRev('/') + 1)); // get item after last slash
}

void DlgStatus::remoteListDoubleClicked(QListViewItem *item) {
	// perms bitfield lists as (sticky) directory
	if (item->text(4)[0] == 'd' || item->text(4)[0] == 't')
		emit syncViewDir(item->text(0));
	else
		emit syncGetFile(item->text(0));
}

void DlgStatus::changeStatus(QString s) {
	statusBar->clear();
	statusBar->message(s);
}

void DlgStatus::clearRecvList() {
	rightList->clear();
	int pos = siteInfo->getRemote().find('/');
	// HPV fixed this shit (2005-12-18)
	if (-1 != pos && (siteInfo->getRemote().length() - 1) != pos)
		addListItem(rightList, "d---------", "", "", "..");
	remoteSelectChanged();
}

void DlgStatus::clearTransList() {
	transferList->clear();
}

int DlgStatus::setCurrFile(QString file) {
	bool matched = false;
	if (((currTransfer = transferList->firstChild()) == transferList->lastItem())
			&& currTransfer->text(0) == file)
		matched = true;
	else {
		for (currTransfer = transferList->firstChild(); currTransfer; currTransfer = currTransfer->nextSibling()) {
			if (currTransfer->text(0) == file) {
				matched = true; // found a match, all good
				break;
			}
		}
	}
	transferList->ensureItemVisible(currTransfer);
	if (matched)
		return 0;
	else
		return 1; // failures shouldn't happen, but...
}

void DlgStatus::refreshLocal() {
	fillLocalList();
}

void DlgStatus::localDirEntered() {
	if (!localPathBox->text())
		return;
	QFileInfo info(localPathBox->text());
	if (info.exists())
		if (info.isDir())
			if (info.isReadable()) {
				setCWD(info.absFilePath());
				refreshLocal();
				return;
			}
			else
				QMessageBox::warning(this, "No Access (QSync)", QString("The directory \"%1\" is not readable.  Directory has not been changed.").arg(localPathBox->text()), QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
		else
			QMessageBox::warning(this, "Not A Directory (QSync)", QString("The entry \"%1\" is not a directory.  Directory has not been changed.").arg(localPathBox->text()), QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
	else
		QMessageBox::warning(this, "Does Not Exist (QSync)", QString("The directory \"%1\" does not exist.  Directory has not been changed.").arg(localPathBox->text()), QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
	// we only fall through here if errors are met.  reset path box.
	localPathBox->setText(localDir->absPath());
}

void DlgStatus::remoteDirEntered() {
}

void DlgStatus::setCWD(QString dir) {
	if ("" != dir) {
		localDir->setPath(dir);
	}
	else {
		localDir->setPath(originalDir);
	}
	fillLocalList();
}

QString DlgStatus::getCWD() {
	return localDir->absPath();
}

void DlgStatus::setFileProgress(QString data) {
	QString percent = data.section(' ', 1, 1, QString::SectionSkipEmpty);
	fileProgress->setProgress(QString(percent).remove('%').toInt());
	currTransfer->setText(1, data.section(' ', 0, 0, QString::SectionSkipEmpty)); // size
	currTransfer->setText(2, percent); // percent, duh
	currTransfer->setText(3, data.section(' ', 2, 2, QString::SectionSkipEmpty)); // eta/time
	currTransfer->setText(4, data.section(' ', 3, 3, QString::SectionSkipEmpty)); // speed
}

void DlgStatus::makeFileComplete() {
	currTransfer->setText(2, "100");
}

void DlgStatus::downloadClicked() {
	emit syncGetFile(rightList->selectedItem()->text(0));
}

void DlgStatus::uploadClicked() {
	emit syncSendFile(leftList->selectedItem()->text(0));
}

void DlgStatus::remoteSelectChanged() {
	if (rightList->selectedItem())
		btnDownload->setEnabled(true);
	else
		btnDownload->setEnabled(false);
}

// some more shit pulled from old main dialog
void DlgStatus::updateInfo(SiteInfo &si) {
	siteInfo = &si;
	hostnameBox->setText(siteInfo->getHost().c_str());
	//remoteEdit->setText(siteInfo->getRemote().c_str());
	//localEdit->setText(siteInfo->getLocalDir().c_str());
	usernameBox->setText(siteInfo->getUser().c_str());
	passwordBox->setText(siteInfo->getPass().c_str());
}

void DlgStatus::changeLocal(const QString &s) {
	// this isn't used right now because syncprocess calls getCWD.  is that good?  dunno.
	// siteInfo->setLocalDir((const char *)s);
	setCWD(s);
}

void DlgStatus::changeHost(const QString &s) {
	siteInfo->setHost((const char *)s);
}

void DlgStatus::changePort(const QString &s) {
	siteInfo->setPort((const char *)s);
}

void DlgStatus::changeUser(const QString &s) {
	siteInfo->setUser((const char *)s);
}

void DlgStatus::changePass(const QString &s) {
	siteInfo->setPass((const char *)s);
}

void DlgStatus::changeRemote(const QString &s) {
	siteInfo->setRemote((const char *)s);
	syncProcess->setNoModuleSpecified((s ? false : true));
}

// FIXME:  
void DlgStatus::remoteChosen(const QString s) {
	remotePathBox->setText(s); // changeRemote() is called via signal
	// call changeRemote manually, since no remote field for a while now.
	//changeRemote(s);
	runSync();
}

// TODO: eventually allow file selection after verifying that source is file, not dir.
void DlgStatus::getDestination() {
	QFileDialog destDlg;
	QString dir;
	if ((dir = destDlg.getExistingDirectory(NULL, this, "dest selection dlg", "Select Destination Directory")))
		localPathBox->setText(dir);
		//changeLocal(dir); // called manually, since no local field for a while now.
}

// TOFIX: SOURCE NEEDS DIR SEL TOO, NOT ONLY FILE!
// TOFIX: BOTH Source/Dest CANNOT BE FILLED WIHTOUT LOCALHOST... RIGHT?
	
// is this even used anymore? this used to be the local file selection dialog.
// TODO: eventually allow multiple file selection... needs implementation elsewhere too.
void DlgStatus::getSource() {
	QFileDialog sourceDlg;
	QString dir;
	if ((dir = sourceDlg.getExistingDirectory(NULL, this, "src selection dlg", "Select Source Directory")))
		remotePathBox->setText(dir);
		//changeRemote(dir); // called manually, since no remote field for a while now.
}

void DlgStatus::runSync() {
	if (hostnameBox->text().length() && portBox->text().length()) {
		// obsolete, this is that dialog now: this->parentWidget()->parentWidget()->hide();
		//statusDlg->show();
		syncProcess->go(RSyncStuff::ChangeDir);
		refreshLocal();
	}
	else
		QMessageBox::critical(this, "Specify Host and Port (QSync)",
				"You must specify a hostname or IP address in the host field\n"
				"(example: rsync.kernel.org or 204.152.191.37), and a port in\n"
				"the port field (default: 873).", QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
}
