#ifndef DLGSTATUS_H
#define DLGSTATUS_H

#include "siteinfo.h"
#include "syncprocess.h"
#include "dlgmodulelist.h"
#include <qwidget.h>
#include <qlistview.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qheader.h>
#include <qdir.h>
#include <qprogressbar.h>
#include <qpushbutton.h>
#include <qstatusbar.h>
#include <qvbox.h>
#include <qsplitter.h>
#include <qstringlist.h>
#include <qevent.h>
#include <qlineedit.h>

class DlgStatus : public QWidget {
		Q_OBJECT
	public:
		DlgStatus(SiteInfo &si, QWidget *parent=0, const char *name=0);
		void addListItem(QListView *list, const char *perms, const char *size, const char *modtime, const char *name, bool dateWeird = false);
		void addTransferItem(QString name);
		void changeStatus(QString s);
		void clearRecvList();
		void clearTransList();
		void setCWD(QString dir);
		QString getCWD();
		void makeFileComplete();
		int setCurrFile(QString file);
	public slots:
		void downloadClicked();
		void setFileProgress(QString data);
		void refreshLocal();
		void uploadClicked();
		void updateInfo(SiteInfo &si);
	signals:
		void syncViewDir(QString s);
		void syncGetFile(QString s);
		void syncSendFile(QString s);
	protected:
		void customEvent(QCustomEvent *e);
		void resizeEvent(QResizeEvent *e);
		void moveEvent(QMoveEvent *e);
	private:
		// data
		DlgModuleList *moduleListDlg;
		QDir *localDir;
		SiteInfo *siteInfo;	// all data for current site
		SyncProcess *syncProcess;	// actually does the work
		// funcshizzles
		void fillLocalList();
		void recenterModDlg();
		// widgets/etc
		QGridLayout *gridMain, *gridLists;
		QHBoxLayout *connFieldBox;
		QPixmap *imgDownload;
		QPixmap *imgUpload;
		QListView *leftList, *rightList, *transferList;
		QListViewItem *currTransfer;
		QProgressBar *fileProgress;
		QPushButton *btnDownload, *btnUpload, *btnStart;
		QSplitter *splitterMain;
		QString nameHeader, permsHeader, originalDir;
		QStatusBar *statusBar;
		QLabel *hostnameLabel, *portLabel, *usernameLabel, *passwordLabel, *localLabel, *remoteLabel;
		QLineEdit *hostnameBox, *portBox, *usernameBox, *passwordBox, *localPathBox, *remotePathBox;
		QVBox *operationsPanel;
		QWidget *listPanel, *topListCont, *connFieldPanel;
	private slots:
		void localListDoubleClicked(QListViewItem *item);
		void remoteListDoubleClicked(QListViewItem *item);
		void remoteSelectChanged();
		void changeLocal(const QString &s);
		void changeHost(const QString &s);
		void changePort(const QString &s);
		void changePass(const QString &s);
		void changeRemote(const QString &s);
		void changeUser(const QString &s);
		void getDestination();
		void getSource();
		void localDirEntered();
		void remoteChosen(QString s);
		void remoteDirEntered();
		void runSync();
};

#endif /* DLGSTATUS_H */
