#include <iostream>
#include <qapplication.h>
#include <qevent.h>
#include "customevents.h"
#include "syncprocess.h"
#include "syncthread.h"
#include "dlgstatus.h"

using namespace std;

extern "C" int rsync_main(int argc,char *argv[]);

/* the function pointers blow are wrappers for some C++ functions, so
	 rsync's C code can call the functions to update the gui with any
	 transfer/listing/etc info.  the dialog instance pointers are set
	 in the DlgMain constructor below.
 */

extern DlgStatus *statusDlgPtr;
extern SyncProcess *SyncProcessPtr;
extern SiteInfo *siteInfoPtr;
extern SyncThread *syncThreadPtr;

// adds remote files listed by the rsync server
void qsAddListItem2(char *perms, char *size, char *modtime, char *name) {
	QStringList *args = new QStringList();
	QCustomEvent *ce = new QCustomEvent(EVENT_ADD_REMOTE);
	args->append(perms);
	args->append(size);
	args->append(modtime);
	args->append(name);
	ce->setData(args);
	QApplication::postEvent(statusDlgPtr, ce);
}
void (*qsAddListItem)(char *, char *, char *, char *) = &qsAddListItem2;

// adds remote files to the to-download list as they're counted
void qsQueueXferItem2(char *name) {
	QCustomEvent *ce = new QCustomEvent(EVENT_QUEUE_XFER);
	int length = strlen(name) + 1; // (+ null)
	/* hopefully postEvent frees this as it does with ce.  if we just pass
		 in "name", it's overwritten by the time the event is actually
		 processed.  making a new object for each event solves this. */
	char *itemName = (char *)malloc(length);
	strncpy(itemName, name, length);
	itemName[length - 1] = '\0';
	ce->setData(itemName);
	QApplication::postEvent(statusDlgPtr, ce);
}
void (*qsQueueXferItem)(char *) = &qsQueueXferItem2;

// tell transfer dialog the currently transferring file changed
void qsSetCurrentFile2(char *name) {
	QCustomEvent *ce = new QCustomEvent(EVENT_SET_CURRENT);
	int length = strlen(name) + 1; // (+ null)
	/* see "hopefully" above to see why we don't free this. */
	char *itemName = (char *)malloc(length);
	strncpy(itemName, name, length);
	itemName[length - 1] = '\0';
	ce->setData(itemName);
	/*if (!qApp->locked()) {
		qApp->lock();*/
		QApplication::postEvent(statusDlgPtr, ce);
		/*qApp->processEvents();
		qApp->unlock();
	}*/
}
void (*qsSetCurrentFile)(char *) = &qsSetCurrentFile2;

void qsSetProgress2(char *data) {
	QCustomEvent *ce = new QCustomEvent(EVENT_SET_PROGRESS);
	int length = strlen(data) + 1; // (+ null)
	/* see "hopefully" above to see why we don't free this. */
	char *itemData = (char *)malloc(length);
	strncpy(itemData, data, length);
	itemData[length - 1] = '\0';
	ce->setData(itemData);
	QApplication::postEvent(statusDlgPtr, ce);
}
void (*qsSetProgress)(char *) = &qsSetProgress2;

// gathers rsync motd and/or module listing
void qsAddRsyncSpew2(char *s) { SyncProcessPtr->addRsyncSpew(s); }
void (*qsAddRsyncSpew)(char *) = &qsAddRsyncSpew2;

// hands rsync the password when it wants it
char *qsGetSitePass2() {
	return (char *)siteInfoPtr->getPass().c_str();
}
char *(*qsGetSitePass)() = &qsGetSitePass2;

void SyncThread::cleanExit(int eventNum) {
	QCustomEvent *ce = new QCustomEvent(eventNum);
	QApplication::postEvent(syncProcess, ce);
	this->exit();
}

void SyncThread::run() {
	rsync_main(rsyncArgCount, rsyncArgs);
}

void SyncThread::setArgs(int argCount, char *args[]) {
	rsyncArgCount = argCount;
	// TODO: is this following line even right? doublecheck this sometime...
	rsyncArgs = (char **)malloc(argCount * sizeof(char *));
	for (int i = 0; i < argCount; i++)
		rsyncArgs[i] = args[i];
}

void SyncThread::setSyncProcess(SyncProcess *sp) {
	syncProcess = sp;
}
